//
//  ThreadCreator.m
//  ThreadExample: a simple example that shows how to create a thread
//
//  Created by Rene Hexel on 28/11/06.
//  Copyright 2006 Rene Hexel. All rights reserved.
//

#import "ThreadCreator.h"
#include <stdio.h>		// for printf() and fflush()

@implementation ThreadCreator

/*
 * initialise the thread, creating the two locks
 */
- init
{
	if (![super init])
		return nil;

	lock = [NSLock new];
	childRunning = [NSLock new];

	if (lock == nil || childRunning == nil)
		return nil;

	return self;
}


/*
 * release the locks
 */
- (void) dealloc
{
	[lock release];
	[childRunning release];
	
	[super dealloc];
}


/*
 * start the child thread
 */
- startChild
{
	[NSThread detachNewThreadSelector: @selector(childThread:)
				 toTarget: self
			       withObject: @"Hello World!"];

	return self;
}


/*
 * wait for the child to quit (unlock its lock)
 */
- waitForChild
{
	//
	// the child keeps the "childRunning" lock while it is running
	// so simply wait for that lock, then unlock it again
	//
	[childRunning lock];
	[childRunning unlock];

	return self;
}


/*
 * print some output using printf().
 * we use the lock and fflush() to avoid conflicts with the child
 */
- print: (NSString *) output
{
	[lock lock];		// protect the critical printf() section

	    printf("%s\n", [output cString]);
	    fflush(stdout);

	[lock unlock];		// end of critical section

	return self;
}


/*
 * here is the actual child thread
 */
- childThread: someObject
{
	//
	// each thread needs its own Autorelease Pool
	//
	NSAutoreleasePool *pool = [NSAutoreleasePool new];

	//
	// tell the parent that we are running
	//
	[childRunning lock];

	//
	// lets examine the object
	//
	NSString *msg = [NSString stringWithFormat:
					@"Child started with object: %@",
					[someObject description] ];

	[self print: msg];

	//
	// do something "useful"
	//
	int i;
	for (i = 0; i < 10; i++)
	{
		[NSThread sleepUntilDate: [[NSDate date] addTimeInterval: 1.0]];
		[self print: [NSString stringWithFormat:
						@"Child iteration %d", i + 1] ];
	}

	[self print: @"Child is about to exit!"];

	//
	// child is finished release the pool, then let the parent know
	//
	[pool release];
	[childRunning unlock];

	return self;
}


@end
